@@ -7,10 +7,12 @@
 #      ///'       
 #     //
 #    '
+from __future__ import with_statement
 from datetime import datetime
 import logging
 import os
 import re
+import threading
 import time
 
 try:
@@ -28,6 +30,16 @@ try:
 except ImportError:
     mysql = None
 
+__all__ = [
+    'ImproperlyConfigured', 'SqliteDatabase', 'MySQLDatabase', 'PostgresqlDatabase',
+    'asc', 'desc', 'Count', 'Max', 'Min', 'Q', 'Field', 'CharField', 'TextField',
+    'DateTimeField', 'BooleanField', 'FloatField', 'IntegerField', 'PrimaryKeyField',
+    'ForeignKeyField', 'Model',
+]
+
+class ImproperlyConfigured(Exception):
+    pass
+
 if sqlite3 is None and psycopg2 is None and mysql is None:
     raise ImproperlyConfigured('Either sqlite3, psycopg2 or MySQLdb must be installed')
 
@@ -192,25 +204,50 @@ class Database(object):
     - execution of SQL queries
     - creating and dropping tables and indexes
     """
-    def __init__(self, adapter, database, **connect_kwargs):
+    def __init__(self, adapter, database, threadlocals=False, **connect_kwargs):
         self.adapter = adapter
         self.database = database
         self.connect_kwargs = connect_kwargs
+        
+        if threadlocals:
+            self.__local = threading.local()
+        else:
+            self.__local = type('DummyLocal', (object,), {})
+        
+        self._conn_lock = threading.Lock()
     
     def connect(self):
-        self.conn = self.adapter.connect(self.database, **self.connect_kwargs)
+        with self._conn_lock:
+            self.__local.conn = self.adapter.connect(self.database, **self.connect_kwargs)
+            self.__local.closed = False
     
     def close(self):
-        self.adapter.close(self.conn)
+        with self._conn_lock:
+            self.adapter.close(self.__local.conn)
+            self.__local.closed = True
+    
+    def get_conn(self):
+        if not hasattr(self.__local, 'closed') or self.__local.closed:
+            self.connect()
+        return self.__local.conn
+    
+    def get_cursor(self):
+        return self.get_conn().cursor()
     
     def execute(self, sql, params=None, commit=False):
-        cursor = self.conn.cursor()
+        cursor = self.get_cursor()
         res = cursor.execute(sql, params or ())
         if commit:
-            self.conn.commit()
+            self.commit()
         logger.debug((sql, params))
         return cursor
     
+    def commit(self):
+        self.get_conn().commit()
+    
+    def rollback(self):
+        self.get_conn().rollback()
+    
     def last_insert_id(self, cursor, model):
         return self.adapter.last_insert_id(cursor, model)
     
@@ -225,8 +262,8 @@ class Database(object):
                 db_field, ', '.join(self.adapter.get_field_types().keys())
             )
     
-    def create_table(self, model_class):
-        framing = "CREATE TABLE %s (%s);"
+    def create_table(self, model_class, safe=False):
+        framing = safe and "CREATE TABLE IF NOT EXISTS %s (%s);" or "CREATE TABLE %s (%s);"
         columns = []
 
         for field in model_class._meta.fields.values():
@@ -260,6 +297,9 @@ class Database(object):
     
     def get_indexes_for_table(self, table):
         raise NotImplementedError
+    
+    def get_tables(self):
+        raise NotImplementedError
 
 
 class SqliteDatabase(Database):
@@ -270,6 +310,10 @@ class SqliteDatabase(Database):
         res = self.execute('PRAGMA index_list(%s);' % table)
         rows = sorted([(r[1], r[2] == 1) for r in res.fetchall()])
         return rows
+    
+    def get_tables(self):
+        res = self.execute('select name from sqlite_master where type="table" order by name')
+        return [r[0] for r in res.fetchall()]
 
 
 class PostgresqlDatabase(Database):
@@ -283,6 +327,18 @@ class PostgresqlDatabase(Database):
             WHERE c.relname = %s AND c.oid = i.indrelid AND i.indexrelid = c2.oid
             ORDER BY i.indisprimary DESC, i.indisunique DESC, c2.relname""", (table,))
         return sorted([(r[0], r[1]) for r in res.fetchall()])
+    
+    def get_tables(self):
+        res = self.execute("""
+            SELECT c.relname
+            FROM pg_catalog.pg_class c
+            LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
+            WHERE c.relkind IN ('r', 'v', '')
+                AND n.nspname NOT IN ('pg_catalog', 'pg_toast')
+                AND pg_catalog.pg_table_is_visible(c.oid)
+            ORDER BY c.relname""")
+        return [row[0] for row in res.fetchall()]
+
 
 class MySQLDatabase(Database):
     def __init__(self, database, **connect_kwargs):
@@ -292,6 +348,10 @@ class MySQLDatabase(Database):
         res = self.execute('SHOW INDEXES IN %s;' % table)
         rows = sorted([(r[2], r[1] == 0) for r in res.fetchall()])
         return rows
+    
+    def get_tables(self):
+        res = self.execute('SHOW TABLES;')
+        return [r[0] for r in res.fetchall()]
 
 
 class QueryResultWrapper(object):
@@ -339,6 +399,11 @@ class QueryResultWrapper(object):
             raise StopIteration
 
 
+# create
+class DoesNotExist(Exception):
+    pass
+
+
 # semantic wrappers for ordering the results of a `SelectQuery`
 def asc(f):
     return (f, 'ASC')
@@ -794,6 +859,25 @@ class SelectQuery(BaseQuery):
         self._order_by.append(
             (self.query_context, field_or_string, ordering)
         )
+    
+    def exists(self):
+        clone = self.paginate(1, 1)
+        clone.query = '(1) AS a'
+        curs = self.database.execute(*clone.sql())
+        return bool(curs.fetchone())
+    
+    def get(self, *args, **kwargs):
+        try:
+            orig_ctx = self.query_context
+            self.query_context = self.model
+            obj = self.where(*args, **kwargs).paginate(1, 1).execute().next()
+            return obj
+        except StopIteration:
+            raise self.model.DoesNotExist('instance matching query does not exist:\nSQL: %s\nPARAMS: %s' % (
+                self.sql()
+            ))
+        finally:
+            self.query_context = orig_ctx
 
     def parse_select_query(self, alias_map):
         if isinstance(self.query, basestring):
@@ -1020,22 +1104,29 @@ class Field(object):
     db_field = ''
     default = None
     field_template = "%(column_type)s%(nullable)s"
+    _field_counter = 0
+    _order = 0
 
     def get_attributes(self):
         return {}
     
-    def __init__(self, null=False, db_index=False, *args, **kwargs):
+    def __init__(self, null=False, db_index=False, verbose_name=None, *args, **kwargs):
         self.null = null
         self.db_index = db_index
         self.attributes = self.get_attributes()
         self.default = kwargs.get('default', None)
+        self.verbose_name = verbose_name
         
         kwargs['nullable'] = ternary(self.null, '', ' NOT NULL')
         self.attributes.update(kwargs)
+        
+        Field._field_counter += 1
+        self._order = Field._field_counter
     
     def add_to_class(self, klass, name):
         self.name = name
         self.model = klass
+        self.verbose_name = self.verbose_name or re.sub('_+', ' ', name).title()
         setattr(klass, name, None)
     
     def render_field_template(self):
@@ -1196,6 +1287,8 @@ class ForeignKeyField(IntegerField):
         self.name = name + '_id'
         self.model = klass
         
+        self.verbose_name = self.verbose_name or re.sub('_', ' ', name).title()
+        
         if self.related_name is None:
             self.related_name = klass._meta.db_table + '_set'
         
@@ -1232,10 +1325,14 @@ class BaseModelOptions(object):
         self.fields = {}
         self.model_class = model_class
     
+    def get_sorted_fields(self):
+        return sorted(self.fields.items(), key=lambda (k,v): (k == self.pk_name and 1 or 2, v._order))
+    
     def get_field_names(self):
-        fields = [self.pk_name]
-        fields.extend([f for f in sorted(self.fields.keys()) if f != self.pk_name])
-        return fields
+        return [f[0] for f in self.get_sorted_fields()]
+    
+    def get_fields(self):
+        return [f[1] for f in self.get_sorted_fields()]
     
     def get_field_by_name(self, name):
         if name in self.fields:
@@ -1304,11 +1401,15 @@ class BaseModel(type):
             pk = PrimaryKeyField()
             pk.add_to_class(cls, _meta.pk_name)
             _meta.fields[_meta.pk_name] = pk
-            
+
+        _meta.model_name = cls.__name__
                 
         if hasattr(cls, '__unicode__'):
             setattr(cls, '__repr__', lambda self: '<%s: %s>' % (
-                self.__class__.__name__, self.__unicode__()))
+                _meta.model_name, self.__unicode__()))
+
+        exception_class = type('%sDoesNotExist' % _meta.model_name, (DoesNotExist,), {})
+        cls.DoesNotExist = exception_class
         
         return cls
 
@@ -1340,7 +1441,14 @@ class Model(object):
         return dict(pairs)
     
     @classmethod
-    def create_table(cls):
+    def table_exists(cls):
+        return cls._meta.db_table in cls._meta.database.get_tables()
+    
+    @classmethod
+    def create_table(cls, fail_silently=False):
+        if fail_silently and cls.table_exists():
+            return
+
         cls._meta.database.create_table(cls)
         
         for field_name, field_obj in cls._meta.fields.items():
@@ -1385,13 +1493,13 @@ class Model(object):
     def get_or_create(cls, **query):
         try:
             inst = cls.get(**query)
-        except StopIteration:
+        except cls.DoesNotExist:
             inst = cls.create(**query)
         return inst
     
     @classmethod            
     def get(cls, *args, **kwargs):
-        return cls.select().where(*args, **kwargs).paginate(1, 1).execute().next()
+        return cls.select().get(*args, **kwargs)
     
     def get_pk(self):
         return getattr(self, self._meta.pk_name, None)
